#-----------------------------------------------------------------------------
#
#   HXFS 2019 - Playing screen
#
#-----------------------------------------------------------------------------

from __future__ import division
from math import sin, cos, hypot, atan2, floor, pi
from pygame import Rect
from OpenGL import GL
from albow.widget import Widget
from albow.layout import Row, Column
from albow.resource import get_cursor
from albow.image_array import get_image_array
from albow.palette_view import PaletteView
from albow.utils import blit_in_rect
from albow.opengl import GLOrtho
from humerus.main_commands import MainCommandScreen
from vectors import Rotation, point_in_polygon
from game import game

HEX_RADIUS = 200
FLEX_BUTTON_RADIUS = HEX_RADIUS + 45
FLEX_BUTTON_SIZE = 30
COS30 = cos(pi / 6)

TRIVERTS = [
    (0, 0),
    (1, 0),
    (0.5, COS30)
]

FLEX_BUTTON_COLORS = [
    (0, 1, 0),
    (1, 0, 0),
    (1, 1, 0),
]

#-----------------------------------------------------------------------------

class FlexButton(object):

    flex_color = (0, 1, 0)
    antiflex_color = (1, 0, 0)
    outline_color = (0, 0, 0)

    def __init__(self, k):
        self.pat = k
        a = -k * pi / 3
        rot = Rotation(a)
        x = FLEX_BUTTON_RADIUS
        r = FLEX_BUTTON_SIZE
        v0 = rot * (x - r, 0)
        v1 = rot * (x + r / 2, 0)
        v2 = rot * (x + r / 2, -r * COS30)
        v3 = rot * (x + r / 2, r * COS30)
        self.vertices_full = [v0, v2, v3]
        self.vertices_lower =[v0, v2, v1]
        self.vertices_upper = [v0, v1, v3]

    def draw(self, b):
        if b == 1:
            fill_and_outline_polygon(self.vertices_full, self.flex_color, self.outline_color)
        elif b == 2:
            fill_and_outline_polygon(self.vertices_full, self.antiflex_color, self.outline_color)
        elif b == 3:
            fill_and_outline_polygon(self.vertices_lower, self.flex_color, self.outline_color)
            fill_and_outline_polygon(self.vertices_upper, self.antiflex_color, self.outline_color)
    
    def hit(self, p):
        if point_in_polygon(p, self.vertices_lower):
            return 1
        elif point_in_polygon(p, self.vertices_upper):
            return 2
        else:
            return 0

#-----------------------------------------------------------------------------

def flex_button_centre(k):
    a = -k * pi / 3
    r = FLEX_BUTTON_RADIUS
    x = r * cos(a)
    y = r * sin(a)
    return (x, y)

def outline_polygon(vertices):
    GL.glBegin(GL.GL_LINE_LOOP)
    for v in vertices:
        GL.glVertex2fv(v)
    GL.glEnd()

def fill_polygon(vertices):
    GL.glBegin(GL.GL_POLYGON)
    for v in vertices:
        GL.glVertex2fv(v)
    GL.glEnd()

def fill_and_outline_polygon(vertices, fill_color, outline_color):
    GL.glColor3fv(fill_color)
    fill_polygon(vertices)
    GL.glColor3fv(outline_color)
    outline_polygon(vertices)

def draw_pats(pats, face, gl_cull, colors):
    GL.glCullFace(gl_cull)
    GL.glPushMatrix()
    for pat in pats:
        t = pat[0]
        s = t.state
        GL.glColor3fv(colors[t.faces[face]])
        GL.glBegin(GL.GL_POLYGON)
        for i in s:
            GL.glVertex2fv(TRIVERTS[i])
        GL.glEnd()
        GL.glRotatef(-60, 0, 0, 1)
    GL.glPopMatrix()

def outline_pats():
    GL.glPushMatrix()
    GL.glColor3f(0, 0, 0)
    GL.glLineWidth(2)
    for k in xrange(6):
        outline_polygon(TRIVERTS)
        GL.glRotatef(-60, 0, 0, 1)
    GL.glPopMatrix()

def draw_flexagon(flex):
    colors = flex.face_colors
    pats = flex.pats_in_depth_order()
    GL.glPushMatrix()
    GL.glScalef(HEX_RADIUS, HEX_RADIUS, 1)
    GL.glEnable(GL.GL_CULL_FACE)
    draw_pats(pats, 0, GL.GL_BACK, colors)
    draw_pats(pats, 1, GL.GL_FRONT, colors)
    GL.glDisable(GL.GL_CULL_FACE)
    outline_pats()
    GL.glPopMatrix()

def draw_flex_buttons(flexability):
    x = FLEX_BUTTON_RADIUS
    r = FLEX_BUTTON_SIZE
    vertices = [(x - r, 0), (x + r / 2, -r * COS30), (x + r / 2, r * COS30)]
    GL.glLineWidth(2)
    for k in xrange(6):
        f = flexability[k]
        if f:
            GL.glPushMatrix()
            GL.glRotatef(-k * 60, 0, 0, 1)
            GL.glColor3fv(FLEX_BUTTON_COLORS[f - 1])
            fill_polygon(vertices)
            GL.glColor3f(0, 0, 0)
            outline_polygon(vertices)
            GL.glPopMatrix()

#-----------------------------------------------------------------------------

class FlexagonView(GLOrtho):

    gl_bg_color = (0.75, 0.75, 0.75)
    
    tool_cursors = [
        get_cursor("flex_cursor.png"),
        get_cursor("split_cursor.png"),
    ]

    def __init__(self, width, height):
        w = width / 2
        h = height / 2
        GLOrtho.__init__(self, Rect(0, 0, width, height), -w, w, -h, h)
        self.flex_buttons = [FlexButton(k) for k in xrange(6)]

    def gl_draw(self):
        flex = game.state
        self.gl_draw_background()
        draw_flexagon(flex)
        #draw_flex_buttons(flex.corner_flexability)
        for f, b in zip(self.flex_buttons, flex.corner_flexability):
            f.draw(b)
    
    def gl_draw_background(self):
        w = self.width / 2
        h = self.height / 2
        GL.glColor3fv(self.gl_bg_color)
        GL.glBegin(GL.GL_POLYGON)
        GL.glVertex2f(-w, -h)
        GL.glVertex2f( w, -h)
        GL.glVertex2f( w,  h)
        GL.glVertex2f(-w,  h)
        GL.glEnd()

    def mouse_down(self, e):
        p = e.ray[0]
        x = p[0]
        y = p[1]
        self.click(x, y)
    
    def click(self, x, y):
        tool = game.current_tool
        if tool == game.FLEX_TOOL:
            self.click_with_flex_tool(x, y)
        elif tool == game.SPLIT_TOOL:
            self.click_with_split_tool(x, y)
    
#     def click_with_flex_tool(self, x, y):
#         for k in xrange(6):
#             cx, cy = flex_button_centre(k)
#             if hypot(x - cx, y - cy) <= FLEX_BUTTON_SIZE:
#                 self.flex_button_clicked(k)
#                 return
    
    def click_with_flex_tool(self, x, y):
        p = (x, y)
        flex = game.state
        for fb in self.flex_buttons:
            h = fb.hit(p)
            if h:
                k = fb.pat
                b = flex.corner_flexability[k]
                if b == 1 or b & h == 1:
                    flex.flex(k)
                elif b == 2 or b & h == 2:
                    flex.antiflex(k)
                return

#     def flex_button_clicked(self, k):
#         flex = game.state
#         f = flex.corner_flexability[k]
#         if f == 1:
#             flex.flex(k)
#         elif f == 2:
#             flex.antiflex(k)
#         elif f == 3:
#             print "Corner", k, "has ambiguous flexability"
    
    def click_with_split_tool(self, x, y):
        if hypot(x, y) < HEX_RADIUS:
            a = atan2(y, x) * 180 / pi
            i = (-int(floor(a / 60))) % 2
            flex = game.state
            new_color = flex.find_unused_color()
            for k in xrange(i, i + 6, 2):
                print "Splitting top of pat", k
                flex.split_top_of_pat(k, new_color)
            flex.dump()
    
    def get_cursor(self, event):
        return self.tool_cursors[game.current_tool]

#-----------------------------------------------------------------------------

class ToolPalette(PaletteView):

    images = get_image_array("tools.png", 2)
    border_width = 2
    bg_color = (255, 255, 255)
    sel_color = (64, 128, 255)
    highlight_style = 'fill'
    
    def __init__(self):
        n = self.num_items()
        PaletteView.__init__(self, (64, 64), n, 1)
    
    def num_items(self):
        return len(self.images)
    
    def draw_item(self, surface, item_no, rect):
        blit_in_rect(surface, self.images[item_no], rect, 'c')
    
    def click_item(self, item_no, event):
        game.current_tool = item_no
    
    def item_is_selected(self, item_no):
        return game.current_tool == item_no

#-----------------------------------------------------------------------------

class PlayScreen(MainCommandScreen):

    is_gl_container = True
    
    def __init__(self, *args, **kwds):
        MainCommandScreen.__init__(self, *args, **kwds)
        tools = ToolPalette()
        tools.top = 5
        tools.right = self.width - 5
        fv = FlexagonView(self.width, self.height)
        self.add(fv)
        self.add(tools)
